#include <Wire.h>  // Only needed for Arduino 1.6.5 and earlier
#include "SSD1306.h" // alias for `#include "SSD1306Wire.h"`

//
// Include custom images
#include "images.h"

int SDAPIN = 1;
int SCLPIN = 3;

int PINBUTTON1 = 2; // LEFT BUTTON 
int PINBUTTON2 = 0; // RIGHT BUTTON

byte b1, b2; // to read PINBUTTONx


//
// Initialize the OLED display using Wire library
SSD1306 display(0x3c, SDAPIN, SCLPIN);


unsigned long counter =0;

//
// ESP-8266 libraries
#include <ESP8266WiFi.h>
#include <DNSServer.h>
#include <ESP8266WebServer.h>

//
// ESP wifi manager, modified from https://github.com/tzapu/WiFiManager
#include "mywifimanager.h"



//
// data that will be read from EEPROM
String host = "";             // url to call to retrieve questions
String APNAME = "MINIQUIZ";   // AP name when access point mode
String APPASS = "";           // AP password


// wi fi connection status
boolean connesso = false;


// wi fi manager
WiFiManager2b wifiManager;


//
// read a single question from the specified url
String readStringFromUrl(String url) {
  
  clearQuestion();  // clear space question

  anim_wifi(60,18,"...mmm...",2);  // loading animation
  display.display();

  //
  // splitting link in host and page
  // http and https are removed and the request is made
  // just with http
  url.replace("http://","");
  url.replace("https://","");
  String page = "";
  byte p = url.indexOf("/");
  page = url.substring(p);
  String host = url;
  host.replace(page,"");
  if (page=="") page="/";

  const int httpPort = 80;
 

  // Use WiFiClient class to create TCP connections
  WiFiClient client;
  if (!client.connect( host.c_str() , httpPort)) {
    connesso = false;
    return "errore di connessione a " + host;
  }



  // We now create and send a URI request
  // ---------------------------------------------------------------
  
  String request = "GET " + page + " HTTP/1.1\r\n" + "Host: " + host + "\r\n" + "Connection: close\r\n\r\n";
  client.print(request);
  delay(1500);



  // Read all the lines of the reply from server
  // and return the data. Data comes from server in a string beginning with { and ending with }
  String data = "";
  while(!client.available()){
   delay(10); yield();
  }  
  while (client.available()) {
    String line = client.readStringUntil('\n');
    
    if (line.indexOf('}') != -1) {
      line.remove(line.indexOf('}'), 1);
      line.remove(line.indexOf('{'), 1);
      line.remove(line.indexOf('\n'), 1);
      line.remove(line.indexOf('\r'), 1);
      data = line;
    }
  }
  return data;
}


//
// exract data from a string using a serparator and an index
String getValue(String data, char separator, int index)
{
  data = data + separator;
  int found = 0;
  int strIndex[] = {0, -1};
  int maxIndex = data.length() - 1;

  for (int i = 0; i <= maxIndex && found <= index; i++) {
    if (data.charAt(i) == separator || i == maxIndex) {
      found++;
      strIndex[0] = strIndex[1] + 1;
      //strIndex[1] = (i == maxIndex) ? i + 1 : i;
      strIndex[1] = i;
    }
  }

  return found > index ? data.substring(strIndex[0], strIndex[1]) : "";
}






// 
// function to animate loading sprites in images.h
void anim_wifi(int x, int y, String s, byte f) {
  unsigned long t = counter;
  if (f< sizeof(wifi) / sizeof(wifi[0]) ) counter=f;
  //if (f<3) counter=f;
  anim_wifi(x,y,s);
  counter = t;
}
void anim_wifi(int x, int y, String s) {
 display.setColor(BLACK);
 display.fillRect(x-60,y,120,22);
 display.setColor(WHITE);
 display.drawXbm(x, y, 8, 8, wifi[counter % 3]);
 if(s!="") {
   display.setTextAlignment(TEXT_ALIGN_CENTER);
   display.drawString(x+4, y+10, s); 
 }
}








//
// function used to echo debug string at the bottom of the screen
// in the last 10 pixels.
// it is also used from the wifimanager to show some debug and 
// when it's called with the "anim_wifi" string it shows the animation
// for wi-fi loading.
void echo(String s) {
  if(s=="anim_wifi") {
    counter = round((float)millis() / 300);
    anim_wifi(60,18,"WI-FI");
    display.display();
  } else {
    clearQuestion();
    display.setTextAlignment(TEXT_ALIGN_LEFT);
    display.drawString(0, 54, s); 
    display.display();  
    delay(200);
    yield();
    delay(200);
  }
}


//
// function to retrieve the next question
// it could come from web (if connection is ok) or from memory (no wi-fi connection available)
String getQuestion() {
 if (connesso) {
    return readStringFromUrl(host);
 } else {
   // there are 5 questions in memory
   int i=random(0,5 + 1);
   char buffer[255];    // make sure this is large enough for the largest string it must hold

   strcpy_P(buffer, (char*)pgm_read_dword(&(string_table[i]))); // Necessary casts and dereferencing, just copy. 
   return buffer;
 }
}



//
// function that wait a button press to go on
void pressToPlay() {
 b1=1;b2=1;
 while(b1+b2==2) {
  b1 = digitalRead(PINBUTTON1);
  b2 = digitalRead(PINBUTTON2);
  yield();
  delay(50);
 }
 waitForRelease();
}


//
// function that wait the release of both buttons
void waitForRelease() {
   byte c1=1; byte c2=1; byte xloop=0;
   while(xloop==0) {
    c1 = digitalRead(PINBUTTON1);
    c2 = digitalRead(PINBUTTON2);
    if(c1+c2==2) xloop=1;
    yield();
    delay(50);
   } 
}



//
// clear the space in the oled used
// to write out the question
void clearQuestion() {
  display.setColor(BLACK);
  display.fillRect(0,10,128,64);
  display.setTextAlignment(TEXT_ALIGN_CENTER);
  display.setColor(WHITE);
}




void setup() {


  // Initialising the UI will init the display too.
  display.init();
  display.flipScreenVertically();
  display.setFont(ArialMT_Plain_10);

  //
  // buttons (we have connected them to pull up resistors)
  pinMode(PINBUTTON1,INPUT); // LEFT
  pinMode(PINBUTTON2,INPUT); // RIGHT

  //
  // activate debug from wifimanager
  // and set the oled callback to print
  // the debug from wifimanager to oled screen
  wifiManager.setDebugOutput(true); 
  wifiManager.setOLEDCallback(echo);

  //
  // buttons initial values
  // b1 = 1 button left is not pressed, b1 = 0 means button right is pressed
  // (same for b2)
  b1 = 1; b2=1;
  
  display.drawXbm(0,0,120,63, MiniQuiz_Logo_bits);
  display.display();
  
  //
  // wait 3 seconds during logo show
  // if both buttons are pressed it starts
  // access mode to configure wi-fi
  unsigned long timer = millis()+3000;
  while (millis()<timer) {
    b1 = digitalRead(PINBUTTON1);
    b2 = digitalRead(PINBUTTON2);
    yield();
    if(b1+b2==0) timer=0;
  }

  display.clear();
  if(b1+b2==0) {
    //
    // buttons pressed, start access mode
    // but before waits until buttons are released
    waitForRelease();
    bool res = wifiManager.activate_AP_and_wait(APNAME.c_str(), APPASS.c_str());
    if(res) {
      // ok
      echo("ok ap fine");
    } else {
      // fail
      echo("ko");
    }


    // wi-fi connection status
    connesso=false;
    delay(500);
  }
  
  if(b1+b2==1) {
    // one of the two buttons was pressed, not handled
    echo ("comando non previsto");
  }

  // read url to call from EEPROM
  host = wifiManager.eepromGet("host");



}

void loop() {

  display.clear();

  // check for internet connection
  if(!connesso){
    connesso = wifiManager.tryWifiConnection();
    //if(connesso) {
    //  anim_wifi(60,18,"OK"); display.display(); delay(100);
    //  display.clear();
    //}
    display.clear();
  }
  


  // for countdown
  unsigned long c = millis()/1000;
  int d = 0;

  int sec = 60;     // game time
  int score = 0;    // game score
  int partita = 1;  // match flag

  //
  // repeat questions game until time goes down to zero
  while (partita==1) {

    // get the next question
    String s = getQuestion();

    // Split s and set data
    String domanda = getValue(s, '|',0);   domanda.trim();
    String risposta1 = getValue(s,'|',1);  risposta1.trim();
    String risposta2 = getValue(s,'|',2);  risposta2.trim();
    String risultato = "";
    String giustaTemp = getValue(s,'|',3);    giustaTemp.trim();
    int giusta = giustaTemp.toInt();

    clearQuestion();

    //
    // show question with small font if the string is long
    // or bigger font if it is a small question
    if(domanda.length() < 25) {
      display.setFont(ArialMT_Plain_16); 
    } else {
      display.setFont(ArialMT_Plain_10); 
    }
    display.drawStringMaxWidth(64, 12, 128, domanda); 

    //
    // show labels with the two answers
    // near the buttons
    display.setFont(ArialMT_Plain_10);
    display.setTextAlignment(TEXT_ALIGN_LEFT);
    display.drawString(0, 52, risposta1); // left button

    display.setTextAlignment(TEXT_ALIGN_RIGHT);
    display.drawString(128, 52, risposta2); // right button

    // buttons value, remember that
    // 0 happens when button is pressed!
    b1 = 1; b2 = 1; 

    while (sec > 0 && b1+b2==2) {
      // repeat until timer is positive buttons are not pressed
      // when a button is pressed or timer ended it gets out of this loop

      c = millis()/1000;
      if(c!=d) {
        // one second is passed, write the countown
	// and the progress bar
        d=c;
        sec--;
        display.setColor(BLACK);
        display.fillRect(0,0,128,10);
        display.setColor(WHITE);
        display.setTextAlignment(TEXT_ALIGN_LEFT);
        display.drawString(0, 0, (String)sec); 
        display.setTextAlignment(TEXT_ALIGN_RIGHT);
        display.drawString(128, 0, (String)score);
        // draw white time rectangle
        int w = round( ((float)108 / (float)60)  * (float)sec);
        display.fillRect(13,3,w,6);
        display.display();
      }

      // read buttons
      b1 = digitalRead(PINBUTTON1);
      b2 = digitalRead(PINBUTTON2);
      yield();
      delay(50);
      
    }

    //
    // a button was pressed!
    // if b1 and b2 are pressed together b1 wins
    if(b1+b2<2) {
      clearQuestion();
      waitForRelease();

      //
      // left button pressed
      if(b1==0) {
        display.setColor(WHITE);
        display.fillRect(0,52,60,12);
        display.setTextAlignment(TEXT_ALIGN_LEFT);
        display.setColor(BLACK);
        display.drawString(0, 52, risposta1); 
      } else {

        //
        // right button pressed
        if(b2==0) {
          display.setColor(WHITE);
          display.fillRect(68,52,60,12);
          display.setTextAlignment(TEXT_ALIGN_RIGHT);
          display.setColor(BLACK);
          display.drawString(128, 52, risposta2); 
        }      
      }

      //
      // show the result!
      if((b1==0 && giusta==1)  ||  (b2==0 && giusta==2)) {
        risultato = "GIUSTO!";  // RIGHT!
        score++;   // increase score
      } else {
        risultato = "SBAGLIATO!";   // WRONG!
        score--;   // decrease score
      }

      display.setColor(BLACK);
      display.fillRect(110,0,18,10); // clear points on top right of screen

      // write result
      display.setColor(WHITE);
      display.setTextAlignment(TEXT_ALIGN_CENTER);
      display.setFont(ArialMT_Plain_16); 
      display.drawString(64, 24, risultato);
      display.setFont(ArialMT_Plain_10); 
      
      // write score
      display.setTextAlignment(TEXT_ALIGN_RIGHT);
      display.drawString(128, 0, (String)score);

      display.display(); // (show changes)

      // wait a press to continue
      pressToPlay();
      
      // set button values
      b1=1;b2=1;
    }
     
    
    
    // if timer ended, game has finished
    if(sec==0) {

      clearQuestion();

      // show final score
      display.drawStringMaxWidth(64, 12, 128, "Fine! hai fatto " + (String)score);
      display.display();
      pressToPlay(); // wait

      // reset main loop and start a new match
      partita = 0;

    }


  } 
  
}
